#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Drawing the diameter as a line and up and curling from there.

use 5.010;
use strict;
use warnings;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'xy_rotate_minus90','xy_rotate_plus90';
use MyGraphs 'Graph_subtree_depth';
use MyLineSegments;
use MyPrintwrap ':all';


# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 7;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1);
my $diameter = $graph->diameter || 0;
print "  % diameter $diameter\n";

my @diameter_path = MyGraphs::Graph_tree_diameter_path($graph);
if ($diameter_path[0] > $diameter_path[-1]) {
  @diameter_path = reverse @diameter_path;
}
### @diameter_path
$#diameter_path == $diameter or die;


my $segments = MyLineSegments->new;
my %seen;
my @vertices;
my $root = 0;
my $root_node;
sub add_vertex {
  my ($v, $x,$y) = @_;
  if ($v eq $root) {
    $root_node = "\\node (root) at ($x,$y) [my vertex];\n";
  } else {
    push @vertices, "$x,$y";
  }
}
sub add_segment {
  my ($x1,$y1, $x2,$y2) = @_;
  if ($segments->is_traversed($x1,$y1, $x2,$y2)) {
    die "oops, overlapping segment";
  }
  $segments->add($x1,$y1, $x2,$y2);
}
foreach my $x (0 .. $#diameter_path) {
  my $y = 0;
  my $v = $diameter_path[$x];
  $seen{$v} = 1;
  add_vertex($v, $x,$y);
  if ($x > 0) {
    add_segment($x,$y, $x-1,$y);
  }
}

my $side = 0;
foreach my $x (0 .. $#diameter_path) {
  my $v = $diameter_path[$x];
  foreach my $child (grep {!$seen{$_}} $graph->neighbours($v)) {
    my $y = 1;
add_segment($x,$y, $x,$y-1);
    draw($child, $x,$y, 0,1, $side);
    $side ^= 1;
  }
}
sub draw {
  my ($v, $x,$y, $dx,$dy, $side) = @_;
  ### draw: "$v at $x,$y  side $side"
  $seen{$v} = 1;
  add_vertex($v, $x,$y);

  my @children = (grep {!$seen{$_}} $graph->neighbours($v));
  ### num children: scalar(@children)
  die if @children > 2;

  @children = sort  # descending depth of subtree
    {Graph_subtree_depth($graph,$v,$b) <=> Graph_subtree_depth($graph,$v,$a)}
    @children;
  ### sorted: @children
  ### depths: [ map { Graph_subtree_depth($graph,$v,$_) } @children ]

  if (@children >= 1) {
    ### straight: $children[0]
    add_segment($x,$y, $x+$dx,$y+$dy);
    draw($children[0], $x+$dx,$y+$dy, $dx,$dy, $side);
  }
  if (@children >= 2) {
    if ($side) {
      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
    } else {
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }
    ### rot to: "$dx,$dy"
    add_segment($x,$y, $x+$dx,$y+$dy);
    draw($children[1], $x+$dx,$y+$dy, $dx,$dy, $side);
  }
}

$Printwrap = 2;
printwrap("\\draw[my tree] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap($root_node);
printwrap("\\path foreach \\p in {");
$Printwrap = 4;
foreach my $xy (@vertices) {
  printwrap("($xy),");
}
printwrap("} ");
printwrap("{ \\p node[my vertex] };\n");

print "  \\newcommand{\\MyDiameter}{$diameter}\n";
print "% GP-Test-Last  TAdiameter($level) == $diameter\n";
print "  % end generated\n";
exit 0;
