#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::AlternatePaper 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','xy_rotate_plus90','xy_rotate_minus90';
use MyPrintwrap ':all';


my $max_level = 10;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

sub path_xyxy_is_side_boundary {
  my ($path, $x1,$y1, $x2,$y2, $side, $n_limit) = @_;
  if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x2,$y2, $n_limit)) {
    return 0;
  }
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  if ($side eq 'left') {
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  } else {
    ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  }
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  if (path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
      && path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
      && path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit)) {
    return undef;
  } else {
    my @array = ($x1,$y1, $x2,$y2, $x2s,$y2s, $x1s,$y1s);
    for (;;) {
      my $mx = min($array[0], $array[2], $array[4], $array[6]);
      my $my = min($array[1], $array[3], $array[5], $array[7]);
      last if $mx == $array[0] && $my == $array[1];
      push @array, splice @array,0,2;
    }
    return \@array;
  }
}

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my @Rs;
my @Ls;
my @Bs;
my @RQs;
my @LQs;
my @BQs;
foreach my $level (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my %count = (left => 0, right => 0);
  my %seen;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    my $x2 = $x + $dx + $rx;
    my $y2 = $y + $dy + $ry;
    foreach my $side ('left','right') {
      my $aref = path_xyxy_is_side_boundary($path, $x,$y, $x+$dx,$y+$dy, $side, $n_hi)
        || next;
      my ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4) = @$aref;
      $count{$side.'_segments'}++;
      next if $seen{"$x1,$y1"}++;
      $count{$side.'_squares'}++;
    }
  }
  push @Ls, $count{'left_segments'};
  push @Rs, $count{'right_segments'};
  push @Bs, $count{'left_segments'} + $count{'right_segments'};
  push @LQs, $count{'left_squares'};
  push @RQs, $count{'right_squares'};
  push @BQs, $count{'left_squares'} + $count{'right_squares'};
}

foreach my $elem (['L',\@Ls],
                  ['R',\@Rs],
                  ['B',\@Bs],
                  ['LQ',\@LQs],
                  ['RQ',\@RQs],
                  ['BQ',\@BQs]) {
  my ($name,$aref) = @$elem;
  $Printwrap = 0;
  printwrap_indent("% GP-DEFINE  ");
  printwrap("${name}_generated_samples = {[");
  $Printwrap += 2;
  foreach my $i (0 .. $#$aref) {
    my $value = $aref->[$i];
    printwrap($value . ($i<$#$aref?',':''));
  }
  printwrap_indent("% GP-DEFINE  ");
  printwrap("]};\n");
}

print "% end generated\n";
exit 0;
