#!/usr/bin/perl -w

# Copyright 2019 Kevin Hyde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';

use lib $FindBin::Bin;
use MyAlternate 'dir';
use MyPrintwrap ':all';

my $level = 8;

print "  % generated by tools/$FindBin::Script\n";

sub num_bits {
  my ($n) = @_;
  my $k = 0;
  while ($n) { $k++; $n>>=1; }
  return $k;
}
CHECK {
  num_bits(7) == 3 or die;
  num_bits(8) == 4 or die;
}

sub n_to_x {
  my ($n) = @_;
  if ($n==0) { return 0; }
  my $k = num_bits($n);
  my $high = 2**($k-1);
  my $r = $n - $high;
  return ($r ? "$k+$r/$high" : $k);
}
CHECK {
  n_to_x(0) == 0 or die;
  n_to_x(1) == 1 or die;
  n_to_x(2) == 2 or die;
  n_to_x(4) == 3 or die;
}

# plot line
my $y_max = 0;
my $y_min = 0;
my $prev_y = dir(0);
$Printwrap = 2;
printwrap("\\draw (0,$prev_y)");
$Printwrap += 2;
my $n_hi = 2**$level-1;
foreach my $n (1 .. $n_hi) {
  my $y = dir($n);
  my $x = n_to_x($n);
  printwrap("--($x,$prev_y)");
  printwrap("--($x,$y)");
  $prev_y = $y;
  $y_max = max($y,$y_max);
  $y_min = min($y,$y_min);
}
my $x_max = n_to_x($n_hi+1);
printwrap("--($x_max,$prev_y);\n");

print "  \\coordinate (x-max) at ($x_max,0);\n";
print "  \\coordinate (y-max) at (0,$y_max);\n";
print "  \\coordinate (y-min) at (0,$y_min);\n";

# X ticks
# $Printwrap = 2;
# printwrap("\\draw (0,0)");
# $Printwrap += 2;
# foreach my $k (-1 .. $level) {
#   my $n = ($k<0 ? 0 : 2**$k);
#   my $x = n_to_x($n);
#   my $semi = ($k==$level ? ";\n" : "");
#   printwrap(" ($x,0) -- +(x-tick)$semi");
# }

# X numbers
# foreach my $k (-1 .. $level-1) {
#   my $n = ($k<0 ? 0 : 2**$k);
#   my $x = n_to_x($n);
#   print "  \\node at ($x,0) [below right,my x number] {\$$n\$};\n";
# }

# X positions
foreach my $k (-1 .. $level-1) {
  my $n = ($k<0 ? 0 : 2**$k);
  my $x = n_to_x($n);
  print "  \\coordinate (n$n) at ($x,0);\n";
}

# Y ticks and numbers
print "  \\begin{scope}[font=\\scriptsize]\n";
$Printwrap = 4;
printwrap("\\draw ");
$Printwrap += 2;
foreach my $y ($y_min .. $y_max) {
  my $semi = ($y==$y_max ? ";\n" : "");
  printwrap(" (0,$y) -- +(y-tick) node[at start,left] {\$$y\$}$semi");
}
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
